@@ -1,9 +1,11 @@
 mlmmj_boolean("closedlist",
 			  "Closed list",
 			  "If the list is open or closed. If it's closed subscription ".
-			  "and unsubscription via mail is disabled. Also note that ".
-			  "confirmation is disabled too, so the -C option to mlmmj-sub ".
-			  "and mlmmj-unsub is of no use with a closed list.");
+			  "and unsubscription via mail is disabled.");
+
+mlmmj_boolean("closedlistsub",
+			  "Closed for subscription",
+			  "Closed for subscription. Unsubscription is possible.");
 
 mlmmj_boolean("nosubconfirm",
 			  "No subscribe confirmation",
@@ -17,6 +19,10 @@ mlmmj_list("moderators",
 		   "Moderators",
 		   "If the list is moderated, this is the list of moderators.");
 
+mlmmj_list("submod",
+		   "Subscription moderators",
+		   "This is the list of moderators that will approve subscriptions.");
+
 mlmmj_boolean("tocc",
 			  "To: Cc:",
 			  "If this option is set, the list address does not have to be in the To: or Cc: header of the email to the list.");
@@ -31,6 +37,11 @@ mlmmj_boolean("subonlypost",
 			  "If this option is set, only people who are subscribed to the list, are allowed to post to it. ".
 			  "The check is made against the \"From:\" header.");
 
+mlmmj_boolean("modnonsubposts",
+			  "Moderate non-subscriber posts",
+			  "If this option is set and subonlypost is enabled, all postings from ".
+			  "people who are not subscribed to the list will be moderated.");
+
 mlmmj_string("prefix",
 			 "Prefix",
 			 "The prefix for the Subject: line of mails to the list. This will alter the Subject: line, ".
@@ -123,3 +134,20 @@ mlmmj_boolean("nosubonlydenymails",
 			  "No subscribers only deny mails",
 			  "This switch turns off whether mlmmj sends out notification about postings ".
 			  "being rejected due to a subscribers only posting list (see 'subonlypost').");
+
+mlmmj_boolean("nodigesttext",
+			  "No digest text summary",
+			  "This switch turns off whether digest mails will have a text part with a thread ".
+			  "summary.");
+
+mlmmj_boolean("nodigestsub",
+			  "No digest subscribers",
+			  "If this option is set, subscription to the digest version of the mailinglist ".
+			  "will be denied. (Useful if you don't want to allow digests and notify users ".
+			  "about it).");
+
+mlmmj_boolean("nonomailsub",
+			  "No nomail subscribers",
+			  "If this option is set, subscription to the nomail version of the mailinglist ".
+			  "will be denied. (Useful if you don't want to allow nomail and notify users ".
+			  "about it).");
