@@ -1,6 +1,6 @@
 /* Copyright (C) 2004 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: prepstdreply.h,v 1.5 2005/10/09 14:17:52 mmj Exp $
+ * $Id: prepstdreply.h,v 1.6 2006/08/06 08:36:47 xi Exp $
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -30,6 +30,6 @@ char *substitute_one(const char *line, const char *listaddr,
 		 const char *listdelim, size_t datacount, char **data);
 char *prepstdreply(const char *listdir, const char *filename, const char *from,
 		   const char *to, const char *replyto, size_t tokencount,
-		   char **data);
+		   char **data, char *customheaders);
 
 #endif /* PREPSTDREPLY_H */
