@@ -1,6 +1,6 @@
 /* Copyright (C) 2003 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: listcontrol.c,v 1.43 2006/01/04 19:10:24 mmj Exp $
+ * $Id: listcontrol.c,v 1.52 2006/12/10 15:00:47 mortenp Exp $
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -63,6 +63,7 @@ enum ctrl_e {
 	CTRL_BOUNCES,
 	CTRL_MODERATE,
 	CTRL_HELP,
+	CTRL_FAQ,
 	CTRL_GET,
 	CTRL_LIST,
 	CTRL_END  /* end marker, must be last */
@@ -92,27 +93,28 @@ static struct ctrl_command ctrl_commands[] = {
 	{ "bounces",            1 },
 	{ "moderate",           1 },
 	{ "help",               0 },
+	{ "faq",                0 },
 	{ "get",                1 },
 	{ "list",               0 }
 };
 
 
 int listcontrol(struct email_container *fromemails, const char *listdir,
-		const char *controladdr, const char *mlmmjsub,
+		const char *controlstr, const char *mlmmjsub,
 		const char *mlmmjunsub, const char *mlmmjsend,
 		const char *mlmmjbounce, const char *mailname)
 {
-	char *atsign, *recipdelimsign, *listdelim, *bouncenr, *tmpstr;
-	char *controlstr, *param = NULL, *conffilename, *moderatefilename;
+	char *bouncenr, *tmpstr;
+	char *param = NULL, *conffilename, *moderatefilename;
 	char *c, *archivefilename, *sendfilename;
 	const char *subswitch;
-	size_t len;
 	struct stat stbuf;
 	int closedlist, nosubconfirm, tmpfd, noget, i, closedlistsub,
 	    subonlyget = 0;
 	size_t cmdlen;
 	unsigned int ctrl;
 	struct strlist *owners;
+	int owner_idx;
 	
 	/* A closed list doesn't allow subscribtion and unsubscription */
 	closedlist = statctrl(listdir, "closedlist");
@@ -126,18 +128,6 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 	else
 		subswitch = "-C";
 	
-	listdelim = getlistdelim(listdir);
-	recipdelimsign = strstr(controladdr, listdelim);
-	MY_ASSERT(recipdelimsign);
-	atsign = index(controladdr, '@');
-	MY_ASSERT(atsign);
-	len = atsign - recipdelimsign;
-
-	controlstr = mymalloc(len);
-	MY_ASSERT(controlstr);
-	snprintf(controlstr, len, "%s", recipdelimsign + strlen(listdelim));
-	myfree(listdelim);
-
 #if 0
 	log_error(LOG_ARGS, "controlstr = [%s]\n", controlstr);
 	log_error(LOG_ARGS, "fromemails->emaillist[0] = [%s]\n",
@@ -148,8 +138,16 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		if (strncmp(controlstr, ctrl_commands[ctrl].command, cmdlen)
 				== 0) {
 
-			if (ctrl_commands[ctrl].accepts_parameter &&
-					(controlstr[cmdlen] == '-')) {
+			if (ctrl_commands[ctrl].accepts_parameter) {
+				if (controlstr[cmdlen] != '-') {
+					errno = 0;
+					log_error(LOG_ARGS, "Command \"%s\""
+						" requires a parameter, but no"
+						" parameter was given."
+						" Ignoring mail",
+						ctrl_commands[ctrl].command);
+					return -1;
+                                }
 				param = mystrdup(controlstr + cmdlen + 1);
 				MY_ASSERT(param);
 				if (strchr(param, '/')) {
@@ -157,18 +155,22 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 					log_error(LOG_ARGS, "Slash (/) in"
 						" list control request,"
 						" discarding mail");
-					exit(EXIT_SUCCESS);
+					return -1;
 				}
-			} else if (!ctrl_commands[ctrl].accepts_parameter &&
-					(controlstr[cmdlen] == '\0')) {
-				param = NULL;
+
 			} else {
-				/* malformed request, ignore and clean up */
-				unlink(mailname);
-				exit(EXIT_SUCCESS);
+				if (controlstr[cmdlen] != '\0') {
+					errno = 0;
+					log_error(LOG_ARGS, "Command \"%s\""
+						" does not accept a parameter,"
+						" but a parameter was given."
+						" Ignoring mail",
+						ctrl_commands[ctrl].command);
+					return -1;
+				}
+				param = NULL;
 			}
 
-			myfree(controlstr);
 			break;
 
 		}
@@ -176,22 +178,42 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 	
 	/* Only allow mails with bad From: header to be bounce mails */
 	if(fromemails->emailcount != 1 && ctrl != CTRL_BOUNCES) {
-		log_error(LOG_ARGS, "Discarding mail with invalid From: "
+		errno = 0;
+		log_error(LOG_ARGS, "Ignoring mail with invalid From: "
 				"which was not a bounce");
-		unlink(mailname);
-		exit(EXIT_SUCCESS);
+		return -1;
 	}
 
+	/* We only need the control mail when bouncing, to save bounced msg */
+	if(ctrl != CTRL_BOUNCES)
+		unlink(mailname);
+
 	switch (ctrl) {
 
 	/* listname+subscribe-digest@domain.tld */
 	case CTRL_SUBSCRIBE_DIGEST:
-		unlink(mailname);
-		if (closedlist || closedlistsub)
-			exit(EXIT_SUCCESS);
-		if (!strchr(fromemails->emaillist[0], '@'))
-			/* Not a valid From: address, silently ignore */
-			exit(EXIT_SUCCESS);
+		if (closedlist || closedlistsub) {
+			errno = 0;
+			log_error(LOG_ARGS, "A subscribe-digest request was"
+				" sent to a closed list. Ignoring mail");
+			return -1;
+		}
+		if (!strchr(fromemails->emaillist[0], '@')) {
+			/* Not a valid From: address */
+			errno = 0;
+			log_error(LOG_ARGS, "A subscribe-digest request was"
+				" sent with an invalid From: header."
+				" Ignoring mail");
+			return -1;
+		}
+		if (statctrl(listdir, "nodigestsub")) {
+			errno = 0;
+			log_error(LOG_ARGS, "A subcribe-digest request was"
+				" denied");
+			send_help(listdir, fromemails->emaillist[0],
+				mlmmjsend, "nodigest", "sub-deny-digest");
+			return -1;
+		}
 		log_oper(listdir, OPLOGFNAME, "mlmmj-sub: request for digest"
 					" subscription from %s",
 					fromemails->emaillist[0]);
@@ -207,12 +229,28 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 
 	/* listname+subscribe-nomail@domain.tld */
 	case CTRL_SUBSCRIBE_NOMAIL:
-		unlink(mailname);
-		if (closedlist || closedlistsub)
-			exit(EXIT_SUCCESS);
-		if (!strchr(fromemails->emaillist[0], '@'))
-			/* Not a valid From: address, silently ignore */
-			exit(EXIT_SUCCESS);
+		if (closedlist || closedlistsub) {
+			errno = 0;
+			log_error(LOG_ARGS, "A subscribe-nomail request was"
+				" sent to a closed list. Ignoring mail");
+			return -1;
+		}
+		if (!strchr(fromemails->emaillist[0], '@')) {
+			/* Not a valid From: address */
+			errno = 0;
+			log_error(LOG_ARGS, "A subscribe-nomail request was"
+				" sent with an invalid From: header."
+				" Ignoring mail");
+			return -1;
+		}
+		if (statctrl(listdir, "nonomailsub")) {
+			errno = 0;
+			log_error(LOG_ARGS, "A subcribe-nomail request was"
+				" denied");
+			send_help(listdir, fromemails->emaillist[0],
+				mlmmjsend, "nonomail", "sub-deny-nomail");
+			return -1;
+		}
 		log_oper(listdir, OPLOGFNAME, "mlmmj-sub: request for nomail"
 					" subscription from %s",
 					fromemails->emaillist[0]);
@@ -228,12 +266,20 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 
 	/* listname+subscribe@domain.tld */
 	case CTRL_SUBSCRIBE:
-		unlink(mailname);
-		if (closedlist || closedlistsub)
-			exit(EXIT_SUCCESS);
-		if (!strchr(fromemails->emaillist[0], '@'))
-			/* Not a valid From: address, silently ignore */
-			exit(EXIT_SUCCESS);
+		if (closedlist || closedlistsub) {
+			errno = 0;
+			log_error(LOG_ARGS, "A subscribe request was"
+				" sent to a closed list. Ignoring mail");
+			return -1;
+		}
+		if (!strchr(fromemails->emaillist[0], '@')) {
+			/* Not a valid From: address */
+			errno = 0;
+			log_error(LOG_ARGS, "A subscribe request was"
+				" sent with an invalid From: header."
+				" Ignoring mail");
+			return -1;
+		}
 		log_oper(listdir, OPLOGFNAME, "mlmmj-sub: request for regular"
 					" subscription from %s",
 					fromemails->emaillist[0]);
@@ -248,12 +294,15 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 
 	/* listname+subconf-digest-COOKIE@domain.tld */
 	case CTRL_CONFSUB_DIGEST:
-		unlink(mailname);
 		conffilename = concatstr(3, listdir, "/subconf/", param);
 		myfree(param);
 		if((tmpfd = open(conffilename, O_RDONLY)) < 0) {
-			/* invalid COOKIE, silently ignore */
-			exit(EXIT_SUCCESS);
+			/* invalid COOKIE */
+			errno = 0;
+			log_error(LOG_ARGS, "A subconf-digest request was"
+				" sent with a mismatching cookie."
+				" Ignoring mail");
+			return -1;
 		}
 		tmpstr = mygetline(tmpfd);
 		chomp(tmpstr);
@@ -273,12 +322,15 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 
 	/* listname+subconf-nomail-COOKIE@domain.tld */
 	case CTRL_CONFSUB_NOMAIL:
-		unlink(mailname);
 		conffilename = concatstr(3, listdir, "/subconf/", param);
 		myfree(param);
 		if((tmpfd = open(conffilename, O_RDONLY)) < 0) {
-			/* invalid COOKIE, silently ignore */
-			exit(EXIT_SUCCESS);
+			/* invalid COOKIE */
+			errno = 0;
+			log_error(LOG_ARGS, "A subconf-nomail request was"
+				" sent with a mismatching cookie."
+				" Ignoring mail");
+			return -1;
 		}
 		tmpstr = mygetline(tmpfd);
 		chomp(tmpstr);
@@ -298,12 +350,15 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 
 	/* listname+subconf-COOKIE@domain.tld */
 	case CTRL_CONFSUB:
-		unlink(mailname);
 		conffilename = concatstr(3, listdir, "/subconf/", param);
 		myfree(param);
 		if((tmpfd = open(conffilename, O_RDONLY)) < 0) {
-			/* invalid COOKIE, silently ignore */
-			exit(EXIT_SUCCESS);
+			/* invalid COOKIE */
+			errno = 0;
+			log_error(LOG_ARGS, "A subconf request was"
+				" sent with a mismatching cookie."
+				" Ignoring mail");
+			return -1;
 		}
 		tmpstr = mygetline(tmpfd);
 		chomp(tmpstr);
@@ -323,12 +378,20 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 
 	/* listname+unsubscribe-digest@domain.tld */
 	case CTRL_UNSUBSCRIBE_DIGEST:
-		unlink(mailname);
-		if (closedlist)
-			exit(EXIT_SUCCESS);
-		if (!strchr(fromemails->emaillist[0], '@'))
-			/* Not a valid From: address, silently ignore */
-			exit(EXIT_SUCCESS);
+		if (closedlist) {
+			errno = 0;
+			log_error(LOG_ARGS, "An unsubscribe-digest request was"
+				" sent to a closed list. Ignoring mail");
+			return -1;
+		}
+		if (!strchr(fromemails->emaillist[0], '@')) {
+			/* Not a valid From: address */
+			errno = 0;
+			log_error(LOG_ARGS, "An unsubscribe-digest request was"
+				" sent with an invalid From: header."
+				" Ignoring mail");
+			return -1;
+		}
 		log_oper(listdir, OPLOGFNAME, "mlmmj-unsub: %s requests"
 					" unsubscribe from digest",
 					fromemails->emaillist[0]);
@@ -344,12 +407,20 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 
 	/* listname+unsubscribe-nomail@domain.tld */
 	case CTRL_UNSUBSCRIBE_NOMAIL:
-		unlink(mailname);
-		if (closedlist)
-			exit(EXIT_SUCCESS);
-		if (!strchr(fromemails->emaillist[0], '@'))
-			/* Not a valid From: address, silently ignore */
-			exit(EXIT_SUCCESS);
+		if (closedlist) {
+			errno = 0;
+			log_error(LOG_ARGS, "An unsubscribe-nomail request was"
+				" sent to a closed list. Ignoring mail");
+			return -1;
+		}
+		if (!strchr(fromemails->emaillist[0], '@')) {
+			/* Not a valid From: address */
+			errno = 0;
+			log_error(LOG_ARGS, "An unsubscribe-nomail request was"
+				" sent with an invalid From: header."
+				" Ignoring mail");
+			return -1;
+		}
 		log_oper(listdir, OPLOGFNAME, "mlmmj-unsub: %s requests"
 					" unsubscribe from nomail",
 					fromemails->emaillist[0]);
@@ -365,12 +436,20 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 
 	/* listname+unsubscribe@domain.tld */
 	case CTRL_UNSUBSCRIBE:
-		unlink(mailname);
-		if (closedlist)
-			exit(EXIT_SUCCESS);
-		if (!strchr(fromemails->emaillist[0], '@'))
-			/* Not a valid From: address, silently ignore */
-			exit(EXIT_SUCCESS);
+		if (closedlist) {
+			errno = 0;
+			log_error(LOG_ARGS, "An unsubscribe request was"
+				" sent to a closed list. Ignoring mail");
+			return -1;
+		}
+		if (!strchr(fromemails->emaillist[0], '@')) {
+			/* Not a valid From: address */
+			errno = 0;
+			log_error(LOG_ARGS, "An unsubscribe request was"
+				" sent with an invalid From: header."
+				" Ignoring mail");
+			return -1;
+		}
 		log_oper(listdir, OPLOGFNAME, "mlmmj-unsub: %s requests"
 					" unsubscribe from regular list",
 					fromemails->emaillist[0]);
@@ -385,14 +464,21 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 
 	/* listname+unsubconf-digest-COOKIE@domain.tld */
 	case CTRL_CONFUNSUB_DIGEST:
-		unlink(mailname);
-		if (closedlist)
-			exit(EXIT_SUCCESS);
+		if (closedlist) {
+			errno = 0;
+			log_error(LOG_ARGS, "An unsubconf-digest request was"
+				" sent to a closed list. Ignoring mail");
+			return -1;
+		}
 		conffilename = concatstr(3, listdir, "/unsubconf/", param);
 		myfree(param);
 		if((tmpfd = open(conffilename, O_RDONLY)) < 0) {
-			/* invalid COOKIE, silently ignore */
-			exit(EXIT_SUCCESS);
+			/* invalid COOKIE */
+			errno = 0;
+			log_error(LOG_ARGS, "An unsubconf-digest request was"
+				" sent with a mismatching cookie."
+				" Ignoring mail");
+			return -1;
 		}
 		tmpstr = mygetline(tmpfd);
 		close(tmpfd);
@@ -412,14 +498,21 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 
 	/* listname+unsubconf-nomail-COOKIE@domain.tld */
 	case CTRL_CONFUNSUB_NOMAIL:
-		unlink(mailname);
-		if (closedlist)
-			exit(EXIT_SUCCESS);
+		if (closedlist) {
+			errno = 0;
+			log_error(LOG_ARGS, "An unsubconf-nomail request was"
+				" sent to a closed list. Ignoring mail");
+			return -1;
+		}
 		conffilename = concatstr(3, listdir, "/unsubconf/", param);
 		myfree(param);
 		if((tmpfd = open(conffilename, O_RDONLY)) < 0) {
-			/* invalid COOKIE, silently ignore */
-			exit(EXIT_SUCCESS);
+			/* invalid COOKIE */
+			errno = 0;
+			log_error(LOG_ARGS, "An unsubconf-nomail request was"
+				" sent with a mismatching cookie."
+				" Ignoring mail");
+			return -1;
 		}
 		tmpstr = mygetline(tmpfd);
 		close(tmpfd);
@@ -439,14 +532,21 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 
 	/* listname+unsubconf-COOKIE@domain.tld */
 	case CTRL_CONFUNSUB:
-		unlink(mailname);
-		if (closedlist)
-			exit(EXIT_SUCCESS);
+		if (closedlist) {
+			errno = 0;
+			log_error(LOG_ARGS, "An unsubconf request was"
+				" sent to a closed list. Ignoring mail");
+			return -1;
+		}
 		conffilename = concatstr(3, listdir, "/unsubconf/", param);
 		myfree(param);
 		if((tmpfd = open(conffilename, O_RDONLY)) < 0) {
-			/* invalid COOKIE, silently ignore */
-			exit(EXIT_SUCCESS);
+			/* invalid COOKIE */
+			errno = 0;
+			log_error(LOG_ARGS, "An unsubconf request was"
+				" sent with a mismatching cookie."
+				" Ignoring mail");
+			return -1;
 		}
 		tmpstr = mygetline(tmpfd);
 		close(tmpfd);
@@ -491,14 +591,28 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 	/* listname+moderate-COOKIE@domain.tld */
 	case CTRL_MODERATE:
 		/* TODO Add accept/reject parameter to moderate */
-		unlink(mailname);
 		moderatefilename = concatstr(3, listdir, "/moderation/", param);
+
+		/* Subscriber moderation */
+		if(strncmp(param, "subscribe", 9) == 0) {
+			log_oper(listdir, OPLOGFNAME, "%s moderated %s",
+				fromemails->emaillist[0], moderatefilename);
+			execlp(mlmmjsub, mlmmjsub,
+					"-L", listdir,
+					"-m", param,
+					"-c", (char *)NULL);
+		}
+
 		sendfilename = concatstr(2, moderatefilename, ".sending");
-		myfree(param);
 
 		if(stat(moderatefilename, &stbuf) < 0) {
 			myfree(moderatefilename);
-			exit(EXIT_SUCCESS); /* just exit, no mail to moderate */
+			/* no mail to moderate */
+			errno = 0;
+			log_error(LOG_ARGS, "A moderate request was"
+				" sent with a mismatching cookie."
+				" Ignoring mail");
+			return -1;
 		}
 		/* Rename it to avoid mail being sent twice */
 		if(rename(moderatefilename, sendfilename) < 0) {
@@ -519,34 +633,69 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 
 	/* listname+help@domain.tld */
 	case CTRL_HELP:
-		unlink(mailname);
-		if(strchr(fromemails->emaillist[0], '@')) {
-			log_oper(listdir, OPLOGFNAME, "%s requested help",
-				fromemails->emaillist[0]);
-			send_help(listdir, fromemails->emaillist[0],
-				  mlmmjsend);
+		if(!strchr(fromemails->emaillist[0], '@')) {
+			/* Not a valid From: address */
+			errno = 0;
+			log_error(LOG_ARGS, "A help request was"
+				" sent with an invalid From: header."
+				" Ignoring mail");
+			return -1;
 		}
+		log_oper(listdir, OPLOGFNAME, "%s requested help",
+				fromemails->emaillist[0]);
+		send_help(listdir, fromemails->emaillist[0], mlmmjsend, "help", "listhelp");
 		break;
 
+       /* listname+faq@domain.tld */
+        case CTRL_FAQ:
+               if(!strchr(fromemails->emaillist[0], '@')) {
+                       /* Not a valid From: address */
+                       errno = 0;
+                       log_error(LOG_ARGS, "A faq request was"
+                               " sent with an invalid From: header."
+                               " Ignoring mail");
+                       return -1;
+               }
+               log_oper(listdir, OPLOGFNAME, "%s requested faq",
+                               fromemails->emaillist[0]);
+               send_help(listdir, fromemails->emaillist[0], mlmmjsend, "faq", "listfaq");
+               break;
+
 	/* listname+get-INDEX@domain.tld */
 	case CTRL_GET:
-		unlink(mailname);
 		noget = statctrl(listdir, "noget");
-		if(noget)
-			exit(EXIT_SUCCESS);
+		if(noget) {
+			errno = 0;
+			log_error(LOG_ARGS, "A get request was sent to a list"
+				" with the noget option set. Ignoring mail");
+			return -1;
+		}
 		subonlyget = statctrl(listdir, "subonlyget");
-		if(subonlyget)
-			if(is_subbed(listdir, fromemails->emaillist[0]) != 0)
-				exit(EXIT_SUCCESS);
+		if(subonlyget) {
+			if(is_subbed(listdir, fromemails->emaillist[0]) != 0) {
+				errno = 0;
+				log_error(LOG_ARGS, "A get request was sent"
+					" from a non-subscribed address to a"
+					" list with the subonlyget option set."
+					" Ignoring mail");
+				return -1;
+			}
+		}
 		/* sanity check--is it all digits? */
 		for(c = param; *c != '\0'; c++) {
-			if(!isdigit((int)*c))
-				exit(EXIT_SUCCESS);
+			if(!isdigit((int)*c)) {
+				errno = 0;
+				log_error(LOG_ARGS, "The get request contained"
+					" non-digits in index. Ignoring mail");
+				return -1;
+			}
 		}
 		archivefilename = concatstr(3, listdir, "/archive/",
 						param);
-		if(stat(archivefilename, &stbuf) < 0)
-			exit(EXIT_SUCCESS);
+		if(stat(archivefilename, &stbuf) < 0) {
+			log_error(LOG_ARGS, "Unable to open archive file");
+			exit(EXIT_FAILURE);
+		}
 		log_oper(listdir, OPLOGFNAME, "%s got archive/%s",
 				fromemails->emaillist[0], archivefilename);
 		execlp(mlmmjsend, mlmmjsend,
@@ -562,7 +711,7 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 
 	/* listname+list@domain.tld */
 	case CTRL_LIST:
-		unlink(mailname);
+		owner_idx = -1;
 		owners = ctrlvalues(listdir, "owner");
 		for(i = 0; i < owners->count; i++) {
 			if(strcmp(fromemails->emaillist[0],
@@ -570,13 +719,29 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 				log_oper(listdir, OPLOGFNAME,
 						"%s requested sub list",
 				fromemails->emaillist[0]);
-				send_list(listdir, owners->strs[i], mlmmjsend);
+				owner_idx = i;
+				break;
 			}
 		}
+		if (owner_idx == -1) {
+			errno = 0;
+			log_error(LOG_ARGS, "A list request was sent to the"
+				" list from a non-owner address."
+				" Ignoring mail");
+			return -1;
+		} else {
+			send_list(listdir, owners->strs[owner_idx], mlmmjsend);
+		}
 		break;
-	}
 
-	unlink(mailname);
+	/* listname+???@domain.tld */
+	default:
+		errno = 0;
+		log_error(LOG_ARGS, "Unknown command \"%s\". Ignoring mail",
+								controlstr);
+		return -1;
+
+	}
 
 	return 0;
 }
