@@ -1,6 +1,6 @@
 /* Copyright (C) 2002, 2003 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: mlmmj-sub.c,v 1.46 2006/01/17 11:49:06 mmj Exp $
+ * $Id: mlmmj-sub.c,v 1.52 2006/09/03 20:55:41 mmj Exp $
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -47,6 +47,202 @@
 #include "statctrl.h"
 #include "prepstdreply.h"
 #include "memory.h"
+#include "ctrlvalues.h"
+#include "chomp.h"
+
+void moderate_sub(const char *listdir, const char *listaddr,
+		const char *listdelim, const char *subaddr,
+		const char *mlmmjsend, enum subtype typesub)
+{
+	int i, fd;
+	char *a = NULL, *queuefilename, *from, *listname, *listfqdn, *str;
+	char *modfilename, *randomstr, *mods, *to, *replyto, *moderators = NULL;
+	char *modfilebase;
+	struct strlist *submods;
+	pid_t pid;
+	char *maildata[6] = { "subaddr", NULL, "moderateaddr", NULL,
+				"moderators", NULL };
+
+	/* generate the file in moderation/ */
+	switch(typesub) {
+		default:
+		case SUB_NORMAL:
+			str = concatstr(4, subaddr, "\n", "SUB_NORMAL", "\n");
+			break;
+		case SUB_DIGEST:
+			str = concatstr(4, subaddr, "\n", "SUB_DIGEST", "\n");
+			break;
+		case SUB_NOMAIL:
+			str = concatstr(4, subaddr, "\n", "SUB_NOMAIL", "\n");
+			break;
+	}
+	
+	randomstr = random_str();
+	modfilename = concatstr(3, listdir, "/moderation/subscribe",
+			randomstr);
+	myfree(randomstr);
+
+	fd = open(modfilename, O_RDWR|O_CREAT|O_EXCL, S_IRUSR|S_IWUSR);
+	while(fd < 0 && errno == EEXIST) {
+		myfree(modfilename);
+		randomstr = random_str();
+		modfilename = concatstr(3, listdir, "/moderation/subscribe",
+				randomstr);
+		myfree(randomstr);
+		fd = open(modfilename, O_RDWR|O_CREAT|O_EXCL, S_IRUSR|S_IWUSR);
+	}
+	if(fd < 0) {
+		log_error(LOG_ARGS, "could not create %s"
+				"ignoring request: %s", str);
+		exit(EXIT_FAILURE);
+	}
+	if(writen(fd, str, strlen(str)) < 0) {
+		log_error(LOG_ARGS, "could not write to %s"
+				"ignoring request: %s", str);
+		exit(EXIT_FAILURE);
+	}
+	
+	close(fd);
+	
+	myfree(str);
+
+	submods = ctrlvalues(listdir, "submod");
+	mods = concatstr(2, listdir, "/control/submod");
+	/* check to see if there's adresses in the submod control file */
+	for(i = 0; i < submods->count; i++)
+		a = strchr(submods->strs[i], '@');
+
+	/* no addresses in submod control file, use owner */
+	if(a == NULL) {
+		/* free the submods struct from above */
+		for(i = 0; i < submods->count; i++)
+			myfree(submods->strs[i]);
+		myfree(submods->strs);
+		myfree(submods);
+		submods = ctrlvalues(listdir, "owner");
+		myfree(mods);
+		mods = concatstr(2, listdir, "/control/owner");
+	}
+
+	/* send mail to moderators about request pending */
+	listdelim = getlistdelim(listdir);
+	listfqdn = genlistfqdn(listaddr);
+	listname = genlistname(listaddr);
+	modfilebase = mybasename(modfilename);
+
+	from = concatstr(4, listname, listdelim, "owner@", listfqdn);
+	to = concatstr(3, listname, "-moderators@", listfqdn);
+	replyto = concatstr(6, listname, listdelim, "moderate-", modfilebase,
+			"@", listfqdn);
+	myfree(modfilebase);
+	for(i = 0; i < submods->count; i++) {
+		printf("%s", submods->strs[i]);
+		str = moderators;
+		moderators = concatstr(3, moderators, submods->strs[i], "\n");
+		myfree(str);
+	}
+
+	maildata[1] = mystrdup(subaddr);
+	maildata[3] = replyto;
+	maildata[5] = moderators;
+
+	queuefilename = prepstdreply(listdir, "submod-moderator",
+				"$listowner$", to, replyto, 3, maildata, NULL);
+	
+	myfree(maildata[1]);
+	
+	/* we need to exec more than one mlmmj-send */
+	
+	pid = fork();
+
+	if(pid < 0) {
+		log_error(LOG_ARGS, "Could not fork for mlmmj-send"
+				"Request in %s not served", modfilename);
+		exit(EXIT_FAILURE);
+	}
+	if(pid == 0) {
+		execl(mlmmjsend, mlmmjsend,
+				"-a",
+				"-l", "4",
+				"-L", listdir,
+				"-s", mods,
+				"-F", from,
+				"-R", replyto,
+				"-m", queuefilename, (char *)NULL);
+		log_error(LOG_ARGS, "execl() of '%s' failed", mlmmjsend);
+	}
+	
+	myfree(to);
+	myfree(replyto);
+	myfree(moderators);
+	
+	/* send mail to requester that the list is submod'ed */
+
+	from = concatstr(4, listname, listdelim, "bounces-help@", listfqdn);
+	queuefilename = prepstdreply(listdir, "submod-requester", "$listowner$",
+					subaddr, NULL, 0, NULL, NULL);
+	
+	myfree(listname);
+	myfree(listfqdn);
+	execl(mlmmjsend, mlmmjsend,
+				"-l", "1",
+				"-L", listdir,
+				"-T", subaddr,
+				"-F", from,
+				"-m", queuefilename, (char *)NULL);
+	log_error(LOG_ARGS, "execl() of '%s' failed", mlmmjsend);
+}
+
+void getaddrandtype(const char *listdir, const char *modstr,
+		char **addrptr, enum subtype *subtypeptr)
+{
+	int fd;
+	char *readaddr, *readtype, *modfilename;
+		
+	modfilename = concatstr(3, listdir, "/moderation/", modstr);
+
+	fd = open(modfilename, O_RDONLY);
+	if(fd < 0) {
+		log_error(LOG_ARGS, "Could not open %s", modfilename);
+		exit(EXIT_FAILURE);
+	}
+
+	readaddr = mygetline(fd);
+	readtype = mygetline(fd);
+
+	close(fd);
+
+	if(readaddr == NULL || readtype == NULL) {
+		log_error(LOG_ARGS, "Could not parse %s", modfilename);
+		exit(EXIT_FAILURE);
+	}
+	
+	chomp(readaddr);
+	*addrptr = readaddr;
+
+	if(strncmp(readtype, "SUB_NORMAL", 10) == 0) {
+		*subtypeptr = SUB_NORMAL;
+		goto freedone;
+	}
+
+	if(strncmp(readtype, "SUB_DIGEST", 10) == 0) {
+		*subtypeptr = SUB_DIGEST;
+		goto freedone;
+	}
+
+	if(strncmp(readtype, "SUB_NOMAIL", 10) == 0) {
+		*subtypeptr = SUB_NOMAIL;
+		goto freedone;
+	}
+
+	log_error(LOG_ARGS, "Type %s not valid in %s", readtype,
+			modfilename);
+
+freedone:
+	myfree(readtype);
+	unlink(modfilename);
+	myfree(modfilename);
+}
 
 void confirm_sub(const char *listdir, const char *listaddr,
 		const char *listdelim, const char *subaddr,
@@ -76,7 +272,7 @@ void confirm_sub(const char *listdir, const char *listaddr,
 	}
 
 	queuefilename = prepstdreply(listdir, listtext, "$helpaddr$",
-				     subaddr, NULL, 0, NULL);
+				     subaddr, NULL, 0, NULL, NULL);
 	MY_ASSERT(queuefilename);
 	myfree(listtext);
 
@@ -123,7 +319,7 @@ void notify_sub(const char *listdir, const char *listaddr,
 	}
 
 	queuefilename = prepstdreply(listdir, listtext, "$listowner$",
-				"$listowner$", NULL, 1, maildata);
+				"$listowner$", NULL, 1, maildata, NULL);
 	MY_ASSERT(queuefilename)
 	myfree(listtext);
 	myfree(maildata[1]);
@@ -210,7 +406,7 @@ void generate_subconfirm(const char *listdir, const char *listaddr,
 	maildata[3] = mystrdup(confirmaddr);
 
 	queuefilename = prepstdreply(listdir, listtext, "$helpaddr$", subaddr,
-				     confirmaddr, 2, maildata);
+				     confirmaddr, 2, maildata, NULL);
 
 	myfree(maildata[1]);
 	myfree(maildata[3]);
@@ -230,7 +426,7 @@ void generate_subconfirm(const char *listdir, const char *listaddr,
 
 static void print_help(const char *prg)
 {
-	printf("Usage: %s -L /path/to/list -a john@doe.org "
+	printf("Usage: %s -L /path/to/list [-a john@doe.org | -m str]"
 	       "[-c] [-C] [-h]\n       [-L] [-d | -n] [-s] [-U] [-V]\n"
 	       " -a: Email address to subscribe \n"
 	       " -c: Send welcome mail\n"
@@ -238,12 +434,13 @@ static void print_help(const char *prg)
 	       " -d: Subscribe to digest of list\n"
 	       " -h: This help\n"
 	       " -L: Full path to list directory\n"
-	       " -n: Subscribe to no mail version of list\n"
-	       " -s: Don't send a mail to the subscriber if already subscribed\n"
+	       " -m: moderation string\n"
+	       " -n: Subscribe to no mail version of list\n", prg);
+	printf(" -s: Don't send a mail to subscriber if already subscribed\n"
 	       " -U: Don't switch to the user id of the listdir owner\n"
 	       " -V: Print version\n"
 	       "When no options are specified, subscription silently "
-	       "happens\n", prg);
+	       "happens\n");
 	exit(EXIT_SUCCESS);
 }
 
@@ -261,7 +458,7 @@ void generate_subscribed(const char *listdir, const char *subaddr,
 	myfree(listdelim);
 
 	queuefilename = prepstdreply(listdir, "sub-subscribed", "$helpaddr$",
-				     subaddr, NULL, 0, NULL);
+				     subaddr, NULL, 0, NULL, NULL);
 	MY_ASSERT(queuefilename);
 
 	myfree(listaddr);
@@ -284,8 +481,9 @@ int main(int argc, char **argv)
 	char *listaddr, *listdelim, *listdir = NULL, *address = NULL;
 	char *subfilename = NULL, *mlmmjsend, *bindir, chstr[2], *subdir;
 	char *subddirname = NULL, *sublockname, *lowcaseaddr;
+	char *modstr = NULL;
 	int subconfirm = 0, confirmsub = 0, opt, subfilefd, lock, notifysub;
-	int changeuid = 1, status, digest = 0, nomail = 0, i = 0;
+	int changeuid = 1, status, digest = 0, nomail = 0, i = 0, submod = 0;
 	int groupwritable = 0, sublock, sublockfd, nogensubscribed = 0, subbed;
 	size_t len;
 	struct stat st;
@@ -301,7 +499,7 @@ int main(int argc, char **argv)
 	mlmmjsend = concatstr(2, bindir, "/mlmmj-send");
 	myfree(bindir);
 
-	while ((opt = getopt(argc, argv, "hcCdnsVUL:a:")) != -1) {
+	while ((opt = getopt(argc, argv, "hcCdm:nsVUL:a:")) != -1) {
 		switch(opt) {
 		case 'a':
 			address = optarg;
@@ -321,6 +519,9 @@ int main(int argc, char **argv)
 		case 'L':
 			listdir = optarg;
 			break;
+		case 'm':
+			modstr = optarg;
+			break;
 		case 'n':
 			nomail = 1;
 			break;
@@ -335,12 +536,21 @@ int main(int argc, char **argv)
 			exit(0);
 		}
 	}
-	
-	if(listdir == 0 || address == 0) {
-		fprintf(stderr, "You have to specify -L and -a\n");
+
+	if(listdir == NULL) {
+		fprintf(stderr, "You have to specify -L\n");
 		fprintf(stderr, "%s -h for help\n", argv[0]);
 		exit(EXIT_FAILURE);
 	}
+	
+	if(address == NULL && modstr == NULL) {
+		fprintf(stderr, "You have to specify -a or -m\n");
+		fprintf(stderr, "%s -h for help\n", argv[0]);
+		exit(EXIT_FAILURE);
+	}
+
+	if(modstr)
+		getaddrandtype(listdir, modstr, &address, &typesub);
 
 	if(strchr(address, '@') == NULL) {
 		log_error(LOG_ARGS, "No '@' sign in '%s', not subscribing",
@@ -457,6 +667,9 @@ int main(int argc, char **argv)
 	}
 	subbed = is_subbed_in(subddirname, address);
 	listdelim = getlistdelim(listdir);
+	if(modstr == NULL)
+		submod = statctrl(listdir, "submod");
+	
 	if(subbed) {
 		if(subconfirm) {
 			close(subfilefd);
@@ -466,6 +679,14 @@ int main(int argc, char **argv)
 			generate_subconfirm(listdir, listaddr, listdelim,
 					    address, mlmmjsend, typesub);
 		} else {
+			if(submod) {
+				close(subfilefd);
+				close(sublockfd);
+				unlink(sublockname);
+				myfree(sublockname);
+				moderate_sub(listdir, listaddr, listdelim,
+					address, mlmmjsend, typesub);
+			}
 			lseek(subfilefd, 0L, SEEK_END);
 			len = strlen(address);
 			address[len] = '\n';
